//LIBRARIES
#include <TimeLib.h>
#include <ESP8266WiFi.h>
#include <WiFiUdp.h>								
#include <Adafruit_NeoPixel.h>

//MY INCLUDES (DIVIDE AND CONCUR)
#include "wifi.h"
#include "ntp.h"
#include  "display.h"
   
boolean colon = false;									
int min1, min5, hr1, hr5;									//clock display values
time_t prevDisplay = 0;										//last display value

//SETUP
void setup() {
		Serial.begin(115200);
		delay(500);
		Serial.println("start");
		Serial.println("Connecting to WiFi");
		wifiSetup();
		Udp.begin(localPort);
		//debug Serial.print("Local port: ");
		//debug Serial.println(Udp.localPort());
		Serial.println("waiting for synchronization");
		while (timeStatus() == timeNotSet){
			setSyncProvider(getNtpTime);
		}
		//synchronisation interval
		setSyncInterval(86400);
		pixels.begin();
		pixels.show();
}
 
void loop(){
	if (timeStatus() != timeNotSet) {					//if status time is not set	
		if (now() != prevDisplay) { 							//if now is not equal to prev display value 
			//debug Serial.print(hour());					//update the display only if time has changed
			//debug printDigits(minute());								
			//debug printDigits(second());
			//debug Serial.println();
			if (colon){
				colon = false;
			}else{
				colon = true;
			}
			//debug Serial.println(colon);
			min1 = minute() % 5;
			min5 = minute() / 5;
			int uur = (hour() - getDST(day(), month(), weekday()));
			if (uur == -1){
				uur = 23;
			}
			hr1 = uur % 5;
			hr5 = uur / 5;
			displayClock(colon, min1, min5, hr1, hr5, day(), month(), year());
			//debug printSerialClock();
			delay(500);
		}
	}
}

//get day light savings time
int getDST(int day, int month, int dow){
  Serial.print("uur: ");
  Serial.println(hour());
  //debug Serial.println(day);
  //debug Serial.println(month);
  //debug Serial.println(dow);
  //0 is wintertijd
  //1 is zomertijd
  //als maand < 3 (voor maart) of > 10 (na oktober) dan is het wintertijd
	if (month < 3 || month > 10)  return 0;
  //als maand > 3 (na maart) of < 10 (voor oktober) dan is het zomertijd
	if (month > 3 && month < 10)  return 1;
  //vorige zondag is dag - dayOfWeek, stel 11-10-2023
  //dag is 11, dayOfWeek is 3 ==>
  //vorige zondag is 11 - 3 = 8 
	int previousSunday = day - (dow + 6);
  //debug Serial.println(previousSunday);
  //eind maart geldt als vorige zondag >= 25 is het al zomertijd
	if ((month == 3) && (previousSunday >= 25))	{
    return 1;
	} else {
	 //eind oktober geldt als vorige zondag > 25 dan is het nog zomertijd
   if ((month == 10) && (previousSunday < 25)) {
      return 1;
    }else{
      return 0;
	  }
  } 
	return false; // this line never gonna happend
}
 
//generates leading zero´s when value below 2 digits
void printDigits(int digits)
{
	Serial.print(":");
	if (digits < 10)
			Serial.print('0');
	Serial.print(digits);
}

//Displays like Mengelehreuhr like serial display
void printSerialClock(){
	for (int i = 0; i < hr5; i++){
		Serial.print("x");
	}
	Serial.println();
	for (int i = 0; i < hr1; i++){
		Serial.print("x");
	}
	Serial.println();
	for (int i = 0; i < min5; i++){
		Serial.print("x");
	}
	Serial.println();
	for (int i = 0; i < min1; i++){
		Serial.print("x");
	}
	Serial.println();
}
